
#include <iostream>
#include <fstream>
#include <string>

#define SHOW(X) cout << #X "=" << (X) << endl

//
// This template class has an fstream as a  data member.
// The method find() performs a binary search of the fstream.
//
template <class T>
class BinSearch {
	fstream fs;
	int nelements;
public:
	explicit BinSearch(const string fname);
	int size();
	void showFile();
	T get(int index);
	int find(const T &t);
};

// Constructor taking a file name
//
template <class T>
BinSearch<T>::BinSearch(string fname) {
	fs.open(fname.c_str(), ios::in | ios::binary);
	if (!fs)
		throw "file open failed";
	fs.seekg(0, ios::end);
	int nbytes = fs.tellg();
	nelements = nbytes / sizeof(T);
	// SHOW(nelements);
}

// Return the number of elements in the file
//
template <class T>
int BinSearch<T>::size() {
	return nelements;
}

template <class T>
void BinSearch<T>::showFile() {
	for (int i=0; i<nelements; i++) {
		cout << get(i) << " ";
	}
	cout << "\n";
}

//
//	Perform a binary search.
//
//	Returns an index to an element in the file
//	which matches the parameter, or -1 if not found.
//	The file is assumed to be sorted.
//
template <class T>
int BinSearch<T>::find(const T &t) {

	int low = 0;
	int high = nelements - 1;
	int middle;

	while (low <= high) {
		middle = (low + high) / 2;
		T midval = get(middle);
		if (t < midval)
			high = middle - 1;		// search lower part
		else if (t > midval)
			low = middle + 1;
		else
			return middle;
	}

	//cout << "BinSearch<T>::find: " << t << " not found\n";
	return -1;
}

template <class T>
T BinSearch<T>::get(int i) {

	fs.seekg(i*sizeof(T));
	T temp;
	fs.read((char *)&temp, sizeof(T));
	if (!fs.good()) 
		throw "file read error";
	return temp;
}
