
#include <vector.h>
#include "BinSearch.h"
#include "Cls.h"

//
// This program tests the BinSearch class
//

// Use a BinSearch object to
// look for a given key, and display results
//
template <class T>
void lookFor(BinSearch<T> &b, const T &key) {
	int index = b.find(key);
	cout << "Search for " << key << ": ";
	if (index == -1)
		cout << "not found";
	else
		cout << "index = " << index;
	cout << endl;
}

// get a line of text from console.
// a zero-length string is acceptable.
//
string getStringNullOK() {
  	const int maxline = 128;
  	char input[maxline];
  	cin.getline(input, maxline, '\n'); 
  	string istr = input;
  	return istr;
}

main()
{
  try {

  	// Get the file name.
  	//
	string fileName("test");
  	cout << "Type the name of the file to be written [default=\"" << 
  		fileName << "\"]: ";
  	string istr = getStringNullOK();
  	if (istr.size() != 0)
		fileName = istr;
	cout << "file name = '" << fileName << "'\n";
	cout << "Press 'Enter' to continue [q=quit]: ";
  	istr = getStringNullOK();
	if (istr == "q") {
		cout << "exiting\n";
		exit(0);
	}

	// Create a file of sorted integers
	//
	int array[] = {2, 3, 5, 7, 11, 13, 17};
	ofstream f(fileName.c_str(), ios::trunc | ios::out | ios::binary);
	if (!f) throw "open failed";
	f.write((char *)array, sizeof(array));
	f.close();

	// perform binary searches
	//
	BinSearch<int> b(fileName);
	cout << endl;
	cout << "File of integers -------------" << endl;
	cout << "Contents of the file (" << b.size() << " elements):\n  ";
	b.showFile();
	lookFor(b,7);
	lookFor(b,2);
	lookFor(b,17);
	lookFor(b,0);
	cout << endl;

	// Create a file of sorted Cls objects.
	// (overwrites the previous data).
	//
	Cls clsArray[] = { 2, 4, 8, 16 };
	f.open(fileName.c_str(), ios::trunc | ios::out | ios::binary);
	if (!f) throw "open failed";
	f.write((char *)clsArray, sizeof(clsArray));
	if (!f.good()) throw "write failed";
	f.close();

	// perform binary searches
	//
	BinSearch<Cls> bc(fileName);
	cout << "File of Cls objects -------------" << endl;
	cout << "Contents of the file (" << bc.size() << " elements):\n  ";
	bc.showFile();
	lookFor(bc,Cls(2));
	lookFor(bc,Cls(16));
	lookFor(bc,Cls(8));
	lookFor(bc,Cls(17));
	cout << endl;
  }
  catch (char *es) {
  	cerr << es << endl;
  	exit(1);
  }
}
